# SASS AST Descriptor Syntax
  
This document specifies the core sass AST node types that support the sass grammar.

[AST Typescript Descriptor](https://github.com/wizardpisces/tiny-sass-compiler/blob/master/parser/ast.ts)

## literal

null

## Expression

```js

PUNC { type: PUNC, value: "(" }           // punctuation: parens((|)), comma(,), semicolon(;) etc.
OPERATOR { type: OPERATOR, value: "!=" }            // + - % * / != ==
TEXT { type: TEXT, value: string }  // TEXT = (TEXT\s+ | TEXT\s+)*
VARIABLE { type: VARIABLE, value: string } // VARIABLE.value === variable's name , expression deleted after evaluation
VAR_KEY { type: NodeTypes.VAR_KEY, value: string } // to solve "TEXT-#{VARIABLE}" , expression replaced after evaluation
PLACEHOLDER {type: NodeTypes.PLACEHOLDER, value: '%TEXT'}
/**
 * https://sass-lang.com/documentation/values/lists
 * any expressions separated with spaces or commas count as a list;
 * iterable eg: $each value in list
  */

LIST { type:NodeTypes.LIST,value:[ TEXT | VARIABLE | VAR_KEY | PUNC | binary ] }
binary { type: NodeTypes.BINARY, operator: op, left: TEXT | VARIABLE | binary, right: TEXT | VARIABLE | binary } // + | - | * | / | %

```

## Statement:

### Normal

```js
body { type:NodeTypes.BODY, chidren:[ Statement ] } // difference between body and child: child contains selector
assign { type: NodeTypes.ASSIGN, left: TEXT | VARIABLE | var_key, right: list } // border : 1px solid red
child { type:NodeTypes.CHILD, selector: TEXT | placeholder | list, children: [ Statement ] }
@import { type: NodeTypes.IMPORT, params:[ TEXT ] }
@include { type: NodeTypes.INCLUDE, id:{ type:"identifier", name: string } , args: [ TEXT | VARIABLE | binary | assign ] }
@extend { type:NodeTypes.EXTEND, param: TEXT | placeholder }
@mixin  { type: NodeTypes.MIXIN, id:{ type:"identifier", name: string } , params: [ VARIABLE | assign ], body: body }
@error  { type: NodeTypes.ERROR, value: list }
```

### Choice

```js
@if { type:NodeTypes.IFSTATEMENT, test: Expression, consequent: body, alternate: IfStatement | body | null }

```

### Loops

```js
@each { type:NodeTypes.EACHSTATEMENT, left: Expression, right: Expression, body:child }
```
## Program

```js
prog { type:NodeTypes.PROGRAM, prog: [ Statement ] }
```
Todos: 

* transform to ts for better documents
* tranform (start,end) to location(line,column,offset)，Error handling to specific position (source filepath , line ,col ,token)
* add keyword @function
* add position to more tokens
* add '( | )' check to binary precedence
* optimize AST Node types
* optimize parser files organization
* reorganize readme.md
* Error recovery
